'generic RAM test for PC-1600
'change constants to adopt for different modules or built in RAM
'data on modules that utilize vert. bank switch cannot be preserved
EQU SADDR &8000   'first addr in Z80 view to test
EQU EADDR &BFFF   'last addr in Z80 view to test
EQU SBANK &02     'first bank in Z80 view to test
EQU EBANK &03     'last bank in Z80 view to test 
EQU SVBNK &00     'first virtual bank to test
EQU EVBNK &01     'last virtual bank to test
EQU PRSRV 0       'flag for data preservation (0 = destructive, i.e. PAT4)  
EQU START &D000


'other constants
EQU CLS &0112
EQU HOME &0109
EQU CRSRSET &0115
EQU UPSCRL &012D
EQU PRTASTR &00EB
EQU BANKSET &0190
EQU TRUE 1
EQU FALSE 0
EQU CR &0D
EQU PAT1 &00   'test pattern1
EQU PAT2 &55   'test pattern2
EQU PAT3 &AA   'test pattern3
EQU PAT4 &FF   'test pattern4
EQU PAGE2 &02  'Z80 page 2


ORG START
CALL CLS
CALL LFEED
'clear log
LD A,0
LD (LOGVBNK),A
LD (LOGBANK),A
LD HL,0
LD (LOGADDR),HL
'run tests
DI
'virtual banks capacity test
LD A,SVBNK 'no vert bank switch?
CP EVBNK
JR Z,SKIPCT
CALL VBTEST
JR C,ENDMAIN 'check for abort
SKIPCT:
'loop over virtual banks
LD A,SVBNK
LD (VBNK),A
LOOPVB:
CALL LFEED
LD A,SVBNK 'no vert bank switch?
CP EVBNK
JR Z,SKIPVB
LD DE,MSGVBSW 'indicate vert bank switch
CALL PRINT
CALL LFEED
CALL SETVBNK
SKIPVB:
'loop over banks
LD A,SBANK
LD (BANK),A
LOOPB:
CALL SETBANK
LD DE,MSGTST
CALL PRINT
'loop over address range
LD BC,SADDR 'BC as pointer
LOOPA: 
CALL TEST
JR C,ENDMAIN 'check for abort
LD HL,EADDR
SBC HL,BC
JR Z,BNKOK
INC BC
JR LOOPA 'addr loop
BNKOK:
LD DE,MSGOK 'indicate test passed for current bank
CALL PRINT
LD A,(BANK)
CP EBANK 'last bank reached?
JR Z,ENDB
INC A
LD (BANK),A
JR LOOPB 'bank loop
ENDB:
LD A,(VBNK)
CP EVBNK 'last virtual bank reached?
JR Z,ENDMAIN
INC A
LD (VBNK),A
JR LOOPVB 'virtual bank loop
ENDMAIN:
EI
RET


VBTEST: 'virtual banks capacity test
LD DE,MSGCTST
CALL PRINT
LD A,SBANK 'set first bank
LD (BANK),A
CALL SETBANK
LD A,SVBNK 'write loop over all virtual banks
LD (VBNK),A
VBL1:
CALL SETVBNK 'write current vbank# to first addr
LD BC,SADDR
LD A,(VBNK) 
LD (BC),A
CP EVBNK 'last virtual bank reached?
JR Z,VBL2
INC A
LD (VBNK),A
JR VBL1
VBL2: 'phase 2
LD A,SVBNK 'verify loop over all virtual banks
LD (VBNK),A
VBL3:
CALL SETVBNK 'read first addr of current vbank# 
LD BC,SADDR
LD HL,VBNK
LD A,(BC)
CP (HL)
JR NZ,VBERR
CP EVBNK 'last virtual bank reached?
JR Z,VBL4
INC A
LD (VBNK),A
JR VBL3
VBL4:
LD DE,MSGOK
CALL PRINT
OR A 'reset carry
JR VBEND
VBERR:
CALL ERROR
VBEND:
RET


TEST: 'tests RAM addr given by (BC) with patterns
OR A 'reset carry
LD A,(BC) 'remember original value
LD (VAL),A
LD A,PAT1 'pattern test
LD (BC),A 
LD A,(BC)
CP PAT1
JR NZ,ERR
LD A,PAT2
LD (BC),A 
LD A,(BC)
CP PAT2
JR NZ,ERR
LD A,PAT3
LD (BC),A 
LD A,(BC)
CP PAT3
JR NZ,ERR
LD A,PAT4
LD (BC),A 
LD A,(BC)
CP PAT4
JR NZ,ERR
LD A,PRSRV 'check preserve flag
CP FALSE
JR Z,TESTEND
LD A,(VAL) 'restore byte
LD (BC),A
JR TESTEND
ERR:
CALL ERROR
TESTEND:
RET


SETVBNK: 'sets current virtual bank to (VBNK)
LD A,(VBNK)
OUT (&28),A
RET


SETBANK: 'sets current bank to (BANK) for PAGE2
PUSH BC
LD B,PAGE2
LD A,(BANK)
CALL BANKSET 
POP BC
RET


ERROR: 'log current pointers
LD A,(VBNK)
LD (LOGVBNK),A
LD A,(BANK)
LD (LOGBANK),A
LD (LOGADDR),BC
CALL LFEED
LD DE,MSGVF
CALL PRINT
SCF 'indicate validation failure
RET


PRINT: 'prints message pointed by DE
LD A,CR
CALL PRTASTR
RET


LFEED: 'line feed
PUSH DE
CALL UPSCRL
LD D,0
LD E,3
CALL CRSRSET
POP DE
RET


'global variables
VBNK: 'current virtual bank
DEFB 0
BANK: 'current bank
DEFB 0
VAL: 'current byte
DEFB 0
LOGADDR: 'log: address of validation error
DEFW 0
LOGBANK: 'log: bank of validation error
DEFB 0
LOGVBNK: 'log: virtual bank of validation error
DEFB 0


MSGCTST:
DEFB "capacity test.."
DEFB CR
MSGTST:
DEFB "test.."
DEFB CR
MSGVBSW:
DEFB "set virtual bank.."
DEFB CR
MSGOK:
DEFB "ok."
DEFB CR
MSGVF:
DEFB "failed!"
DEFB CR

END
